/*******************************************************************************
* File Name: Sensor.c  
* Version 2.10
*
* Description:
*  This file contains API to enable firmware control of a Pins component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "Sensor.h"

#define SetP4PinDriveMode(shift, mode)  \
    do { \
        Sensor_PC =   (Sensor_PC & \
                                (uint32)(~(uint32)(Sensor_DRIVE_MODE_IND_MASK << (Sensor_DRIVE_MODE_BITS * (shift))))) | \
                                (uint32)((uint32)(mode) << (Sensor_DRIVE_MODE_BITS * (shift))); \
    } while (0)


/*******************************************************************************
* Function Name: Sensor_Write
********************************************************************************
*
* Summary:
*  Assign a new value to the digital port's data output register.  
*
* Parameters:  
*  prtValue:  The value to be assigned to the Digital Port. 
*
* Return: 
*  None 
*  
*******************************************************************************/
void Sensor_Write(uint8 value) 
{
    uint8 drVal = (uint8)(Sensor_DR & (uint8)(~Sensor_MASK));
    drVal = (drVal | ((uint8)(value << Sensor_SHIFT) & Sensor_MASK));
    Sensor_DR = (uint32)drVal;
}


/*******************************************************************************
* Function Name: Sensor_SetDriveMode
********************************************************************************
*
* Summary:
*  Change the drive mode on the pins of the port.
* 
* Parameters:  
*  mode:  Change the pins to one of the following drive modes.
*
*  Sensor_DM_STRONG     Strong Drive 
*  Sensor_DM_OD_HI      Open Drain, Drives High 
*  Sensor_DM_OD_LO      Open Drain, Drives Low 
*  Sensor_DM_RES_UP     Resistive Pull Up 
*  Sensor_DM_RES_DWN    Resistive Pull Down 
*  Sensor_DM_RES_UPDWN  Resistive Pull Up/Down 
*  Sensor_DM_DIG_HIZ    High Impedance Digital 
*  Sensor_DM_ALG_HIZ    High Impedance Analog 
*
* Return: 
*  None
*
*******************************************************************************/
void Sensor_SetDriveMode(uint8 mode) 
{
	SetP4PinDriveMode(Sensor__0__SHIFT, mode);
}


/*******************************************************************************
* Function Name: Sensor_Read
********************************************************************************
*
* Summary:
*  Read the current value on the pins of the Digital Port in right justified 
*  form.
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value of the Digital Port as a right justified number
*  
* Note:
*  Macro Sensor_ReadPS calls this function. 
*  
*******************************************************************************/
uint8 Sensor_Read(void) 
{
    return (uint8)((Sensor_PS & Sensor_MASK) >> Sensor_SHIFT);
}


/*******************************************************************************
* Function Name: Sensor_ReadDataReg
********************************************************************************
*
* Summary:
*  Read the current value assigned to a Digital Port's data output register
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value assigned to the Digital Port's data output register
*  
*******************************************************************************/
uint8 Sensor_ReadDataReg(void) 
{
    return (uint8)((Sensor_DR & Sensor_MASK) >> Sensor_SHIFT);
}


/* If Interrupts Are Enabled for this Pins component */ 
#if defined(Sensor_INTSTAT) 

    /*******************************************************************************
    * Function Name: Sensor_ClearInterrupt
    ********************************************************************************
    *
    * Summary:
    *  Clears any active interrupts attached to port and returns the value of the 
    *  interrupt status register.
    *
    * Parameters:  
    *  None 
    *
    * Return: 
    *  Returns the value of the interrupt status register
    *  
    *******************************************************************************/
    uint8 Sensor_ClearInterrupt(void) 
    {
		uint8 maskedStatus = (uint8)(Sensor_INTSTAT & Sensor_MASK);
		Sensor_INTSTAT = maskedStatus;
        return maskedStatus >> Sensor_SHIFT;
    }

#endif /* If Interrupts Are Enabled for this Pins component */ 


/* [] END OF FILE */
